#!/usr/bin/env python3

"""Weaver

"""

__author__ = "Meik Teßmer"
__version__ = "0.2.0"


import os.path
import re
import sys


# map lang hints to comment markers
# currently supported: one-line comments like #, // etc.
lang_map = {
        "bash": "#",
        "python": "#",
        "c": "/\*",
        "cpp": "//",
        "java": "//",
        "go": "//",
        "lua": "--",
        "vim": '"',
        }
# if no language hints can be found, use this
default_lang = "python"


def is_chunk_end(line, mark="@"):
    match = re.match("^" + mark + "[ ]?", line)
    if match:
        return True
    else:
        return False

def get_code_chunk(abspath, given_path, chunk_name, no, lang_hint):
    """extract code chunks from given file"""

    try:
        with open(abspath, "r") as handle:
            # read compelete file if no chunk_name was given
            # prepend <<filename>>, append @
            if chunk_name == "":
                return "<<" + given_path + ">>= (" + lang_hint + ")\n" + handle.read() + "\n@"

            code = []
            chunk_no = -1
            in_code_chunk = False
            comment = lang_map.get(lang_hint, default_lang)
            for line in handle:

                line = line.rstrip()
                # is this the line we searched for?
                name, _ = parse_include_line(line, open_mark=comment + "\s*<<")
                if name and name == chunk_name:
                    # ok, set status and start numbering
                    in_code_chunk = True
                    chunk_no += 1
                    if chunk_no == no:
                        # add line without leading comment and spaces
                        #code.append(line.lstrip(comment).strip() + " (" +
                        #        lang_hint + ")")
                        # rebuild line to avoid errors with C like comments
                        # that needs an */ at the end of the line
                        code.append("<<" + name + ">>= (" + lang_hint + ")")
                    continue

                # if we are in the code chunk, store the line if it is not the
                # closing line and the number is correct
                if in_code_chunk:
                    if not is_chunk_end(line, comment + "@"):
                        # store only if this is the needed chunk no
                        if chunk_no == no:
                            code.append(line)

                        continue
                    else:
                        # otherwise
                        if chunk_no == no:
                            code.append("@\n")
                        in_code_chunk = False
                        continue
    except OSError as err:
        return "Cannot open file: %s" % err
    else:
        return "\n".join(code)


def split_chunk_name(chunk_name):
    """
    Extract path, code chunk name, and chunk no.

    """
    path, name_part = chunk_name.replace("<<", "").replace(">>", "").split(":")

    if "#" in name_part:
        name, no = name_part.split("#")
    else:
        name = name_part
        no = 0

    return path, name, int(no)


def parse_include_line(line, open_mark = "<<", close_mark = ">>="):
    """
    Return code chunk name and optionally a language hint (None).

    """

    # neweb syntax
    match = re.match(open_mark + "([^>]+)" + close_mark + "\s*\((\w+)\)", line)
    if not match:
        # ok, use normal noweb syntax
        match = re.match(open_mark + "([^>]+)" + close_mark, line)
        if match:
            return (match.group(1), None)
        else:
            return None, None
    else:
        return (match.group(1), match.group(2))


def is_include_line(line):
    chunk_name, lang_hint = parse_include_line(line.rstrip())
    if not chunk_name:
        return False
    else:
        return True


def main(doc=None):
    in_ = sys.stdin
    out = sys.stdout

    for line in in_:
        # check for inclusion
        if is_include_line(line):
            # extract path, code chunk name, and chunk no
            chunk_name_part, lang_hint = parse_include_line(line.rstrip())
            given_path, code_chunk_name, chunk_no = split_chunk_name(chunk_name_part)
            abspath = os.path.abspath(given_path)
            code_chunk = get_code_chunk(abspath, given_path, code_chunk_name, chunk_no, lang_hint)
            out.write(code_chunk)
        else:
            out.write(line)


#<<Hauptfunktion>>= (python)
if __name__ == "__main__":
    main()
#@


#<<Hauptfunktion>>= (python)
print()
#@
